#include <iostream>
#include <fstream>
#include <lapacke.h>
#include <gsl/gsl_rng.h>

int main (int argc, char **argv) {
   const lapack_int nrhs = 1;
   std::ofstream out;

   if (argc < 3) {
      std::cerr << "Usage: " << argv[0] << " N  filename " << std::endl;
      std::cerr << "\tWhere N is the number of points to generate, and filename is the output." << std::endl;
      exit(-1);
   }

   lapack_int n = atoi(argv[1]);
   if (n < 4 || n > 10000) {
      int defaultn = 100;
      std::cerr << "Invalid number of points '" << argv[1] << "'." << std::endl;
      std::cerr << "Using " << defaultn << "." << std::endl;
      n = defaultn;
   }
 
   out.open(argv[2]);
   if (out.fail()) {   
      std::cerr << "Cannot open file " << argv[2] << "'." << std::endl;
      exit(-2);
   }

   /* setup b and A*/
   double b[n];
   b[0] = 1.;
   for (int i=1; i<n; i++) 
        b[i] = 0.;

   double a[3][n];
   const int UP=0;
   const int DIAG=1;
   const int LO=2;
   for (int i=0; i<n-1; i++) {
     a[UP]  [i] = -1.;
     a[DIAG][i] =  2.;
     a[LO]  [i] = -1.;
   }
   a[UP]  [0]   = 0.;
   a[DIAG][0]   = 1.;
   a[DIAG][n-1] = 1.;
   a[LO]  [n-2] = 0.;

   lapack_int info = LAPACKE_dgtsv(LAPACK_ROW_MAJOR, n, nrhs, &(a[LO][0]), &(a[DIAG][0]), &(a[UP][0]), &(b[0]), 1);

   if (info != 0) 
       std::cerr << "Error: Info = " << info << std::endl;

   /* solution overwrites ICs */
   out << "#x  Temp" << std::endl;
   for (int i=0; i<n; i++) {
       out << i << "\t" << b[i] << std::endl;
   }

   out.close();
   return 0 ;
}

