#include <cstring>
#include <iostream>
#include <fstream>
#include <lapacke.h>
#include <gsl/gsl_rng.h>

void buildYValues(int n, int seed, double *y) {
    gsl_rng *mt = gsl_rng_alloc(gsl_rng_mt19937);
    gsl_rng_set(mt, seed);
    for (int i=0; i<n; i++) 
        y[i] = gsl_rng_uniform(mt);

    gsl_rng_free(mt);
    return;
}

int main (int argc, char **argv) {

   const lapack_int npow = 4;  /* build a cubic: 4 powers of x (incl 1) */
   const lapack_int nrhs = 1;  /* just one b vector */
   std::ofstream out;

   if (argc < 3) {
      std::cerr << "Usage: " << argv[0] << " N  filename " << std::endl;
      std::cerr << "\tWhere N is the number of points to generate, and filename is the output." << std::endl;
      exit(-1);
   }

   lapack_int n = atoi(argv[1]);
   if (n < npow || n > 10000) {
      int defaultn = 100;
      std::cerr << "Invalid number of points '" << argv[1] << "'." << std::endl;
      std::cerr << "Using " << defaultn << "." << std::endl;
      n = defaultn;
   }
 
   out.open(argv[2]);
   if (out.fail()) {   
      std::cerr << "Cannot open file " << argv[2] << "'." << std::endl;
      exit(-2);
   }

   /* setup x, y, and A */
   double x[n];
   for (int i=0; i<n; i++) 
        x[i] = i;

   double a[n][npow];
   for (int i=0; i<n; i++) {
     a[i][0] = 1.;
     for (int j=1; j<npow; j++) 
        a[i][j] = a[i][j-1]*x[i];
   }

   double y[n], ycopy[n];
   buildYValues(n, 1, &(y[0]));
   for (int i=0; i<n; i++) ycopy[i] = y[i];

   lapack_int info = LAPACKE_dgels(LAPACK_ROW_MAJOR,'N',n,npow,nrhs,&(a[0][0]),npow,&(y[0]),1);

   if (info != 0) 
       std::cerr << "Error: Info = " << info << std::endl;

   out << "# parameters: " << y[0] << "\t" << y[1] <<"\t" <<y[2] << "\t" << y[3] << std::endl;
   out << "#x  y  yfit" << std::endl;
   for (int i=0; i<n; i++) {
       out << x[i] << "\t" << ycopy[i] << "\t" << y[0] + (y[1] + (y[2] + y[3]*x[i])*x[i])*x[i] << std::endl;
   }

   out.close();
   return 0 ;
}

