#include <iostream>
#include <fstream>
#include <lapacke.h>
#include <gsl/gsl_rng.h>

int main (int argc, char **argv) {
   std::ofstream out;

   if (argc < 3) {
      std::cerr << "Usage: " << argv[0] << " N  filename " << std::endl;
      std::cerr << "\tWhere N is the number of points to generate, and filename is the output." << std::endl;
      exit(-1);
   }

   lapack_int n = atoi(argv[1]);
   if (n < 4 || n > 10000) {
      int defaultn = 100;
      std::cerr << "Invalid number of points '" << argv[1] << "'." << std::endl;
      std::cerr << "Using " << defaultn << "." << std::endl;
      n = defaultn;
   }
 
   out.open(argv[2]);
   if (out.fail()) {   
      std::cerr << "Cannot open file " << argv[2] << "'." << std::endl;
      exit(-2);
   }

   /* setup b and A*/
   double diag[n];
   double subdiag[n];
   for (int i=0; i<n; i++) {
     diag   [i] = -2.;
     subdiag[i] =  1.;
   }
   double z[n][n];

   lapack_int info = LAPACKE_dstev( LAPACK_ROW_MAJOR, 'V', n, diag, subdiag, &(z[0][0]), n);

   if (info != 0) 
       std::cerr << "Error: Info = " << info << std::endl;

   /* solution overwrites subdiagonal */
   out << "#e  v[0...n]" << std::endl;
   for (int i=0; i<n; i++) {
       out << diag[i] << "\t";
       for (int j=0; j<n; j++) {
           out << z[i][j] << "\t";
       }
       out << std::endl;
   }

   out.close();
   return 0 ;
}
